/*
 * bn256-20080525/fp6.c 
 * Peter Schwabe
 * Public domain
*/

#include "parameters.h"
#include "fp6.h"

void fp6_init()
{
	fp2e_set_str(xi, BN_XI);
	fp2e_set_str(ypminus1, BN_YPMINUS1);
	fpe_set_str(zeta, BN_ZETA);
	fp2e_set_str(xi2, BN_XI2);
	fp2e_set_str(_1o27xi3, BN_1O27XI3);
	fp2e_set_str(_1o3xi3, BN_1O3XI3);
	fp2e_set_str(_1o3xi, BN_1O3XI);
	fpe_set_str(_1o3modp, BN_1O3MODP); // 1/3 \in \F_p
	
	fpe_set_str(cometa_c0_const, BN_COMETA_C0_CONST);
	fpe_set_str(cometa_c1_const, BN_COMETA_C1_CONST);
}

void fp6_clear()
{
	;
}
