/*
cpucycles sparcestimate.h version 20080104
D. J. Bernstein
Public domain.
*/

#ifndef CPUCYCLES_sparcestimate_h
#define CPUCYCLES_sparcestimate_h

#ifdef __cplusplus
extern "C" {
#endif

extern long long cpucycles_sparcestimate(void);
extern long long cpucycles_sparcestimate_persecond(void);

#ifdef __cplusplus
}
#endif

#ifndef cpucycles_implementation
#define cpucycles_implementation "sparcestimate"
#define cpucycles cpucycles_sparcestimate
#define cpucycles_persecond cpucycles_sparcestimate_persecond
#endif

#endif
