/*
 * bn256-20080525/fp2.h 
 * Peter Schwabe
 * Public domain
*/

#ifndef FP2_H
#define FP2_H

#include <gmp.h>
#include "fp.h"
#include "parameters.h"


void fp2_init(void);
void fp2_clear(void);

#endif // ifdef FP2_H
