/*
 * bn256-20080525/fp.h 
 * Peter Schwabe
 * Public domain
*/

#ifndef FP_H
#define FP_H

#include <gmp.h>

#include "fpe.h"

mpz_t p;
unsigned long p_inv; // -p^{-1} mod 2^{GMP_LIMB_BITS} used in Montgomery reduction

void fp_init(void);

void fp_clear(void);

#endif // ifdef FP_H
