/*
 * bn256-20080525/points.h 
 * Peter Schwabe
 * Public domain
*/

#ifndef POINTS_H
#define POINTS_H

#include "curvepoint_fp.h"
#include "twistpoint_fp2.h"

curvepoint_fp_t curve_gen; // generator of E(\F_p)
twistpoint_fp2_t twist_gen; // generator of the subgroup of order n of E'(\F_{p^2}) 

void points_init(void);

void points_clear(void);

#endif // ifdef POINTS_H
