/*
 * bn256-20080525/fp12.c 
 * Peter Schwabe
 * Public domain
*/

#include "fp2e.h"
#include "fp6e.h"
#include "fp12.h"
#include "parameters.h"

void fp12_init()
{
	fp6e_set_str(tau, BN_TAU);
	fp2e_set_str(zpminus1, BN_ZPMINUS1);
	fp2e_set_str(zpminus1inv, BN_ZPMINUS1INV);
}

void fp12_clear()
{
	;
}
